function  [ax, bx, cx] = mnbrak1(ax,bx,key);
%
%  [ax, bx, cx] = mnbrak1(ax,bx,key);
%
% This script will create brackets required by 
% the brent.m and dbrent.m optimization codes.
%
% inputs:
% ax = distinct point near optimum, scalar
% bx = distinct point near optimum, scalar
% key is either 'root' for locating roots or 'min' for locating minima.
% f(x) entered in the function funkeval.m
%
% outputs:
% 
% ax = distinct point, scalar
% bx = distinct point, scalar
% cx = distinct point, scalar
% such that ax < bx < cx or ax > bx > cx  
% and f(ax) > f(bx) and f(cx) > f(bx)
%
% example usage:
%
% [ax, bx, cx] = mnbrak1(1,2,'min');
%
%  This code is based on the "mnbrak" code of 
%  Press, W.H.,  Flannery, B.P., Teukolsky, S.A., Vetterling, W.T.
%  "Numerical Recipes", Cambridge University Pres, Cambridge, 1986, p. 281-282.
%  
%  Fortran translated to Matlab by David Keffer
%  Department of Chemical Engineering, University of Tennessee
%  original translation date: May, 1999
%  last modified date: April 4, 2013
%
gold = 1.618034;
glimit = 100.0;
tiny=1.e-20;
fa = funkeval_filter(ax,key);
fb = funkeval_filter(bx,key);
if (fb > fa)
   dum = ax;
   ax = bx;
   bx = dum;
   dum = fb;
   fb = fa;
   fa = dum;
end
%
% first guess for c
%
cx = bx+gold*(bx-ax);
fc = funkeval_filter(cx,key);
%
%  main loop
%
while (fb >= fc) 
   %
   % try parabolic fit
   %
   r = (bx - ax)*(fb-fc);
   q = (bx-cx)*(fb-fa);
   u = bx - ( (bx-cx)*q - (bx-ax)*r ) / ( 2*sign(max(abs(q-r),tiny)*(q-r))*abs(q-r) );
   ulim = bx + glimit*(cx - bx);
   if  ( (bx -u)*(u-cx) > 0 )
      fu = funkeval_filter(u,key);
      if ( fu < fc )
         ax = bx;
         fa = fb;
         bx = u;
         fb = fu;
         return;
      elseif (fu > fb )
         cx = u;
         fc = fu;
         return;
      end
      %
      % parabolic fit was no good, try default magnification
      %
      u = cx+gold*(cx-bx);
      fu = funkeval_filter(u,key); 
   elseif ( (cx-u)*(u-ulim) > 0 )
      fu = funkeval_filter(u,key); 
      if ( fu < fc ) 
         bx = cx;
         cx = u;
         u = cx + gold*(cx-bx);
         fb = fc;
         fc = fu;
         fu = funkeval_filter(u,key);
      end
   elseif ( (u-ulim)*(ulim-cx) >= 0) 
      u=ulim;
      fu = funkeval_filter(u,key);
   else
      u = cx + gold*(cx -bx);
      fu = funkeval_filter(u,key);
   end
   ax = bx;
   bx = cx;
   cx = u;
   fa = fb;
   fb = fc;
   fc = fu;
end


         
      



function f = funkeval_filter(x,key);
f = funkeval(x);
if ( strcmp(key,'root') == 1 )
   f = abs(f);
end
