function [x0,err] = nrnd_opt1(x0);
%
%  [x0,err] = nrnd_opt1(x0)
%
% This script will perform optimization by the
% Newton Raphson with Numerical Derivatives method.
%
% f'(x) = 0
%
% inputs:
% x0 = initial guess of x
% f(x) entered in the function "funkeval" at the bottom of this file
%
% outputs:
% 
% x0 = converged solution
% err = relative error on x
%
% example usage:
%
%  [x0,err] = nrnd_opt1(2)
%
% Author:  David Keffer 
% original date: October 22, 1998
% last modified date: March 22, 2013
%

%
%  maximum number of iterations
%
maxit = 100;
%
%  acceptable tolerance
%
tol = 1.0e-6;
%
%  initialize error and counter
%
err = 100.0;
icount = 0;
xold =x0;
%
%  iteration loop
%
while (err > tol & icount <= maxit)
   icount = icount + 1;
   h = min(0.01*xold,0.01);
   f = dfunkeval(xold,h);
   df = d2funkeval(xold,h);
   xnew = xold - f/df;
   if (icount > 1)
      err = abs((xnew - xold)/xnew);
   end
   fprintf(1,'icount = %i xold = %e f = %e df = %e xnew = %e  err = %e \n',icount, xold, f, df, xnew, err);
   xold = xnew;
end
%
%  update guess
%
x0 = xnew;

%
%  if you did not converge, let the user know
%
if (icount >= maxit)
   % you ran out of iterations
   fprintf(1,'Sorry.  You did not converge in %i iterations.\n',maxit);
   fprintf(1,'The final value of x was %e \n', x0);
end

%
%  the function must be entered here
%
function f = funkeval(x)
f = exp(x)-sqrt(x);

%
%  this routine calculates the numerical derivative
%
function df = dfunkeval(x,h)
fp = funkeval(x+h);
fn = funkeval(x-h);
df = (fp - fn)/(2*h);

%
%  this routine calculates the numerical second derivative
%
function d2f = d2funkeval(x,h)
fp = funkeval(x+h);
fo = funkeval(x);
fn = funkeval(x-h);
d2f = (fp - 2*fo + fn)/(h*h);

