function [f,x] = polak(p,tol,iprint,key)
%
%  [f,x] = polak(p,tol,iprint,key)
%
%  This routine is called the Fletcher-Reeves-Polak-Ribiere variant 
%  of the conjugate gradient Methods for finding a minimum in n-dimensions.
%
% inputs:
% p is an initial guess, column vector of length n
% tol is the relative tolerance on x, scalar
% iprint = value of 1 requests iteration information
% key = 'min' for minimization and 'root' for rootfinding
% the objective function is entered in the function funkeval.m 
%
% outputs:
% 
% x = converged solution, column vector of length n
% f = optimized value of function, scalar
%
% example usage:
%
%  [f,x] = polak([1;2],1.0e-6,1,'min')
%
%  This code is based on the "FRPRMN" code of 
%  Press, W.H.,  Flannery, B.P., Teukolsky, S.A., Vetterling, W.T.
%  "Numerical Recipes", Cambridge University Pres, Cambridge, 1986, p. 305-306.
%  
%  Fortran translated to Matlab by David Keffer
%  Department of Chemical Engineering, University of Tennessee
%  original translation date: May,1999
% last modified date: April 5, 2013
%


%
%  check for input argument consistency
%
n = max(size(p));
%
%  parameters
%
maxit = 1000;
eps = 1.0e-10;
g = zeros(n,1);
h = zeros(n,1);
xi = zeros(n,1);
%
%  evaluate function at initial guess
%
fp = funkeval_filter(p,key);
%
%  determine the derivatives of the objective function
%
for i = 1:1:n
   xi(i) = funkdi(p,key,i);
end
%
for j = 1:1:n
   g(j) = -xi(j);
   h(j) = g(j);
   xi(j) = h(j);
end
pold = zeros(n,1);
pold(1:n) = p(1:n);
err = 100.0;
ftol = 1.0;
iter = 0;
%
%  print initial values and slopes
%
   if (iprint == 1)
      fprintf (1, 'iter = %5i error = %9.2e fobj = %15.7e  ', iter, err, fp );
      fprintf (1,'  x ');
      for i = 1:1:n
          fprintf (1,' %15.7e', p(i));
      end   
      fprintf (1,' \n');
      fprintf (1,' dx ');
      for i = 1:1:n
          fprintf (1,' %15.7e', xi(i));
      end   
      fprintf (1,' \n');
   end
%
%  start the main loop
%
%while (err > ftol)
while (err > tol)
   %
   %  do line minimization
   %
   [fret,p,xi] = linmin(p,xi,n,tol,iprint,key);
   %
   %  get function and derivatives at next point
   %
   %fp =  funkeval_filter(p,key);
   for i = 1:1:n
   	xi(i) = funkdi(p,key,i);
   end
   %
   %
   %
   gg = 0.0;
   dgg = 0.0;
   for j = 1:1:n
      gg = gg  + g(j)^2;
   %   dgg = dgg + xi(j)^2;  This statement for Fletcher-Reeves
   %   dgg = dgg + (xi(j)+g(j))*xi(j);  This statement for Polak-Ribiere
   	dgg = dgg + (xi(j)+g(j))*xi(j);
   end
   if ( gg == 0) 
      return
   end
   gam = dgg/gg;
   for j = 1:1:n
      g(j) = -xi(j);
      h(j) = g(j)+gam*h(j);
      xi(j) = h(j);
   end
	%
   %  check iterations
   %
   err = 0.0;
   for i = 1:1:n
       err = err + ((p(i) - pold(i))/p(i))^2;
   end
   err = sqrt(err/n);
   pold(1:n) = p(1:n);
   %err = abs(fp-fret);
   %ftol = 0.5*tol*(abs(fp)+abs(fret) );
   iter = iter +1;
   if (iter > maxit)
      error('maximum number of iterations exceeded \n ');
   end
   fp = fret;
   if (iprint == 1)
      fprintf (1, 'iter = %5i error = %9.2e fobj = %15.7e  \n', iter, err, fp );
      fprintf (1,'  x ');
      for i = 1:1:n
          fprintf (1,' %15.7e', p(i));
      end   
      fprintf (1,' \n');
      fprintf (1,' dx ');
      for i = 1:1:n
          fprintf (1,' %15.7e', xi(i));
      end   
      fprintf (1,' \n');
   end
      fprintf (1, ' %5i %15.7e  %15.7e  %15.7e   %15.7e  \n', iter, p(1:2), fp, err );

end
%
% return solution
%
f = fret;
for j = 1:1:n
   x(j) = p(j);
end


%
%  this function performs the line minization 
%
function  [fret,p,xit] = linmin(p,xit,n,tol,iprint,key);
global pcom xcom ncom
pcom = p;
xcom = xit;
ncom = n;
%
%  create the brackets for the line minimization
%
axi = 0.0;
bxi = 1.0;
p1 = sqrt(sum(p(:).*p(:))/n);
[axi, bxi, cxi] = mnbrakn(axi,bxi,iprint,key);
%if (iprint ==1)
%   fprintf(1,' brackets %15.7e %15.7e %15.7e \n',axi,bxi,cxi); 
%end
%
%  perform the line minimization
%
[fret,xmin] = brentn(axi,bxi,cxi,tol,iprint,key);
%[fret,xmin] = dbrentn(axi,bxi,cxi,tol,iprint,key);
for j = 1:1:n
   xit(j) = xmin*xit(j);
   p(j) = p(j) + xit(j);
end




%
%  change the syseqninput output from a vector to a single function
%
function f = funkeval_filter(x,key);
f = funkeval(x);
if ( strcmp(key,'root') == 1 )
   f = abs(f);
end

%
%  numerically estimate derivative
%
function df = funkdi(x,key,idim)
xorig = x(idim);
dxcon = 0.01;
ddx = min(dxcon*x(idim),dxcon);
i12dx = 1.0/(12.0*ddx);
xp(1) = x(idim) - 2*ddx;
xp(2) = x(idim) - ddx;
xp(3) = x(idim) + ddx;
xp(4) = x(idim) + 2*ddx;
%fprintf (1, 'xp %15.7e %15.7e %15.7e %15.7e \n', xp(1:4));
for j = 1:1:4
   x(idim) = xp(j);
   fp(j) = funkeval_filter(x,key);
end
%fprintf (1, 'fp %15.7e %15.7e %15.7e %15.7e \n', fp(1:4));
df = i12dx*( -fp(4) + 8*fp(3) - 8*fp(2) + fp(1) );


