function [b,bsd,MOF] = linreg1(x,y);
%
% [b,bsd,MOF] = linreg1(x,y);
%
% This script will perform a single variable linear regression of the form
%
% y = b(1) + b(2)*x
%
% inputs:
% x = independent variable, column vector of size n 
% y = dependent variable, column vector of length n
%
% outputs:
% 
% b = regression coefficients, column vector of length 2
% intercept is first element, slope is second element
% bsd = coefficient standard deviations, column vector of length 2 
% MOF = measure of fit, scalar
%
% example usage:
%
% [b,bsd,MOF] = linreg1([1; 2; 3], [4; 5.1; 6])
%
% Author:  David Keffer 
% original date: October 22, 1998
% last modified date: March 20, 2013
%

% 
% Step One.  Determine number of data points
%
n = max(size(x));

%
% Step Two.  Dimension the vectors and matrices of interest
%
b = zeros(2,1);
bsd = zeros(2,1);
yhat = zeros(n,1);

%
%  Step Three.  Compute Sums
%

%
%  average value of x
%
xsum = 0.0;
for i = 1:1:n
   xsum = xsum + x(i);
end
xavg = xsum/n;

%
%  average value of y
%
ysum = 0.0;
for i = 1:1:n
   ysum = ysum + y(i);
end
yavg = ysum/n;

%
%  average value of x^2
%
x2sum = 0.0;
for i = 1:1:n
   x2sum = x2sum + x(i)*x(i);
end
x2avg = x2sum/n;

%
%  variance of x
%
xvarsum = 0.0;
for i = 1:1:n
   xvarsum = xvarsum + (x(i)-xavg)^2;
end

%
%  variance of y
%
yvarsum = 0.0;
for i = 1:1:n
   yvarsum = yvarsum + (y(i)-yavg)^2;
end

%
%  covariance of y
%
xycovarsum = 0.0;
for i = 1:1:n
   xycovarsum = xycovarsum + (x(i)-xavg)*(y(i)-yavg);
end

%
%  Step Four.  Compute Regression Coefficients
%

b(2) = xycovarsum/xvarsum;
b(1) = yavg - b(2)*xavg;

%
%  Step Five.  Compute Standard Deviations of Coefficients
%
SSE = 0.0;
SSR = 0.0;
for i = 1:1:n
   yhat(i) = b(1) + b(2)*x(i);
   SSR = SSR + (yhat(i) - yavg)^2;
   SSE = SSE + (y(i) - yhat(i))^2;
end
SST = SSR + SSE;
MOF = SSR/SST;
sigma = SSE/(n-2);
bsd(2) = sigma/xvarsum;
bsd(1) = x2avg*sigma/xvarsum;
bsd(1:2) = sqrt(bsd(1:2));

%
%  Step Six.  Plot
%
figure(1);
plot(x,y,'ro'), xlabel( 'x' ), ylabel ( 'y' );         
hold on;
plot(x,yhat,'k-');
hold off;

