function [b,bsd,MOF] = polyreg(m,x,y);
%
% [b,bsd,MOF] = polyreg(m,x,y);
%
% This script will perform a polynomial regression of the form
%
% y = b(1) + b(2)*x + b(3)*x^2 + ... + b(m+1)*x^m 
%
% inputs:
% m = order of polynomial 
% x = independent variable, column vector of length n
% y = dependent variable, column vector of length n
%
% outputs:
% 
% b = regression coefficients, column vector of length m+1
% intercept is first element, slope is second element
% bsd = coefficient standard deviations, column vector of length m+1 
% MOF = measure of fit, scalar
%
% example usage:
%
% [b,bsd,MOF] = polyreg(2,[1; 2; 3; 4],[4; 11; 22; 36])
%
% Author:  David Keffer 
% original date: October 22, 1998
% last modified date: March 20, 2013
%

% 
% Step One.  Determine number of data points
%
n = max(size(y));

%
% Step Two.  Dimension the vectors and matrices of interest
%
mp1 = m + 1;
b = zeros(mp1,1);
bsd = zeros(mp1,1);
yhat = zeros(n,1);
g = zeros(mp1,1);
a = zeros(mp1,mp1);

%
% Step Three.  Add column of 1's to x matrix
%
xp1 = ones(n,mp1);
for i = 2:1:mp1
   xp1(1:n,i) = x(1:n).^(i-1);
end
%
%  Step Four.  Create A matrix and g vector
%
gvec = [xp1]'*y;
Amat = [xp1]'*xp1;
detA = det(Amat);
Amatinv = inv(Amat);

b = Amatinv*gvec;

%
%  Step Five.  Compute Standard Deviations of Coefficients
%

dof = n - m;
yhat = xp1*b;
yavg = sum(y)/n;
SSE = 0.0;
SSR = 0.0;
for i = 1:n
   SSR = SSR + (yhat(i) - yavg)^2;
   SSE = SSE + (y(i) - yhat(i))^2;
end
SST = SSR + SSE;
MOF = SSR/SST;
%
s2 = SSE/(dof);
for j = 1:mp1
   bsd(j) =sqrt(Amatinv(j,j)*s2);
end

%
%  Step Six.  Plot
%
figure(1);
plot(x,y,'ro'), xlabel( 'x' ), ylabel ( 'y' );         
hold on;
plot(x,yhat,'k-');
hold off;
